#include "TMath.h"
#include "TFile.h"
#include "RooWorkspace.h"
#include "RooAbsPdf.h"
#include "RooRealVar.h"
#include "RooArgSet.h"
#include "RooArgList.h"

#include <iostream>
using namespace std;

// ================ INSTRUCTIONS:
// initialize the class without an argument. make sure that the RooWorkspace root-file
// is named SUS_16_034_likelihoodPDFs.root and is in the same directory (or adapt the path/name)

// then you can call the function:
//      get_nll (float met, float zpt, float mlb, float ldp)
// to which you should pass the missing momentum, the dilepton-pT, the sum of mlb's, and the leptons-delta phi
// in that exact order!

// if you encounter problems, please contact marc.dunser@cern.ch

class likelihood {

    public:

        likelihood();
        ~likelihood();

        TFile * f_pdfs;
        RooWorkspace * f_w;

        RooAbsPdf * zptpdf;
        RooAbsPdf * metpdf;
        RooAbsPdf * ldppdf;
        RooAbsPdf * mlbpdf;

        RooArgSet * obszpt;
        RooArgSet * obsmet;
        RooArgSet * obsldp;
        RooArgSet * obsmlb;

        float zptPdfVal;
        float metPdfVal;
        float ldpPdfVal;
        float mlbPdfVal;

        float get_nll(float, float, float, float);

        void printSomeRandoms();
    

};

likelihood::~likelihood() {
    f_pdfs->Close();
    std::cout << "good bye" << std::endl;
}

likelihood::likelihood() {
    std::cout << "loading the likelihood class..." << std::endl;

    f_pdfs = TFile::Open("SUS_16_034_likelihoodPDFs.root","READ");
    f_w = (RooWorkspace *) f_pdfs->Get("w");

    zptpdf = f_w->pdf("zpt_analyticalPDF_DA");
    metpdf = f_w->pdf("met_analyticalPDF_DA");
    ldppdf = f_w->pdf("ldp_analyticalPDF_DA");
    mlbpdf = f_w->pdf("mlb_analyticalPDF_DA");

    obszpt = new RooArgSet(*(f_w->var("lepsZPt_Edge" )));
    obsmet = new RooArgSet(*(f_w->var("met_Edge"     )));
    obsldp = new RooArgSet(*(f_w->var("lepsDPhi_Edge")));
    obsmlb = new RooArgSet(*(f_w->var("sum_mlb_Edge" )));

    zptPdfVal = -999.;
    metPdfVal = -999.;
    ldpPdfVal = -999.;
    mlbPdfVal = -999.;
}

float likelihood::get_nll (float met, float zpt, float mlb, float ldp) {
    
    // get the values - dilepton pT
    f_w->var("lepsZPt_Edge")->setVal(zpt);
    zptPdfVal = zptpdf->getVal(obszpt);

    // get the values - MET
    f_w->var("met_Edge")->setVal(met);
    metPdfVal = metpdf->getVal(obsmet);

    // get the values - lepton dPhi
    f_w->var("lepsDPhi_Edge")->setVal(ldp);
    ldpPdfVal = ldppdf->getVal(obsldp);

    // get the values - MET
    f_w->var("sum_mlb_Edge")->setVal(mlb);
    mlbPdfVal = mlbpdf->getVal(obsmlb);

    return -1.*TMath::Log(zptPdfVal*metPdfVal*ldpPdfVal*mlbPdfVal);

}

void likelihood::printSomeRandoms () {

    // call the likelihood like this: likelihood->get_nll(met, dileptonpt, summlb, leptonsdphi)
    // NOTE THAT MET HAS TO BE GREATER THAN 150!!!

    std::cout << "example likelihood discriminator: " << get_nll(242., 142., 442., 0.42) << std::endl;
    std::cout << "example likelihood discriminator: " << get_nll(195., 234., 222., 0.52) << std::endl;
    std::cout << "example likelihood discriminator: " << get_nll(195., 234., 222., 0.52) << std::endl;

}
